import 'package:flutter/widgets.dart';

import 'package:responsive_layout_builder/src/layout_size.dart';
import 'package:responsive_layout_builder/src/screen_size.dart';
import 'package:responsive_layout_builder/src/screen_size_settings.dart';

/// Signature for a function that builds a widget given an [LayoutSize].
///
/// Used by [ResponsiveLayoutBuilder.builder].
typedef Widget ResponsiveLayoutWidgetBuilder(
    BuildContext context, ScreenSize size, Orientation orientation);

/// Builds a widget tree that can depend on the parent widget's [LayoutSize] (orientation and layout).
///
/// Constraints are based on all available space that this widget can occupy, not on screen size
class ResponsiveLayoutBuilder extends StatefulWidget {
  /// Builds the widgets below this widget given this widget's [LayoutSize]
  final ResponsiveLayoutWidgetBuilder builder;

  /// Sets usage of device size or all available space
  ///
  /// Defaults to [true]
  ///
  /// If [false] device size is ommited, and all available space will be used in calculations
  final bool useDeviceScreenSize;

  /// Sets default size if no condition was passed
  ///
  /// Defaults to [LayoutSize.mediumMobile]
  final LayoutSize defaultSize;

  /// Sets screen sizes for different devices which affects on [LayoutSize]
  ///
  /// Defaults to [watch = 220], [smallMobile = 360], [middleMobile = 400], [largeMobile = 600], [smallTablet = 720], [largeTablet = 840]
  final ScreenSizeSettings portraitSizes;

  /// Sets screen sizes for different devices which affects on [LayoutSize]
  ///
  /// Defaults to [watch = 220], [smallMobile = 600], [middleMobile = 720], [largeMobile = 960], [smallTablet = 1024], [largeTablet = 1280]
  final ScreenSizeSettings landscapeSizes;

  /// Creates an responsive layout builder.
  ///
  /// The [builder], [useDeviceScreenSize], [portraitSizes] and [landscapeSizes] argument must not be null.
  const ResponsiveLayoutBuilder({
    Key key,
    @required this.builder,
    this.useDeviceScreenSize = true,
    this.defaultSize = LayoutSize.mobile,
    this.portraitSizes = const ScreenSizeSettings.portrait(),
    this.landscapeSizes = const ScreenSizeSettings.landscape(),
  })  : assert(builder != null),
        assert(portraitSizes != null),
        assert(landscapeSizes != null),
        assert(useDeviceScreenSize != null),
        super(key: key);

  @override
  _ResponsiveLayoutBuilderState createState() =>
      _ResponsiveLayoutBuilderState();
}

class _ResponsiveLayoutBuilderState extends State<ResponsiveLayoutBuilder> {
  MobileLayoutSize _getMobileLayoutSize(
      ScreenSizeSettings sizes, double width) {
    MobileLayoutSize result;

    if (width > sizes.watch && width <= sizes.smallMobile) {
      result = MobileLayoutSize.small;
    } else if (width > sizes.smallMobile && width <= sizes.mediumMobile) {
      result = MobileLayoutSize.medium;
    } else {
      result = MobileLayoutSize.large;
    }

    return result;
  }

  TabletLayoutSize _getTabletLayoutSize(
      ScreenSizeSettings sizes, double width) {
    TabletLayoutSize result;

    if (width > sizes.largeMobile && width <= sizes.smallTablet) {
      result = TabletLayoutSize.small;
    } else {
      result = TabletLayoutSize.large;
    }

    return result;
  }

  ScreenSize _getLayoutSize(ScreenSizeSettings sizes, double width) {
    LayoutSize size;
    MobileLayoutSize mobile;
    TabletLayoutSize tablet;

    if (width <= sizes.watch) {
      size = LayoutSize.watch;
    } else if (width > sizes.watch && width <= sizes.largeMobile) {
      size = LayoutSize.mobile;
      mobile = _getMobileLayoutSize(sizes, width);
    } else if (width > sizes.largeMobile && width <= sizes.largeTablet) {
      size = LayoutSize.tablet;
      tablet = _getTabletLayoutSize(sizes, width);
    } else if (width > sizes.largeTablet) {
      size = LayoutSize.desktop;
    } else {
      size = widget.defaultSize ?? LayoutSize.mobile;
    }

    return ScreenSize(
      size: size,
      mobile: mobile,
      tablet: tablet,
    );
  }

  @override
  Widget build(BuildContext context) {
    return OrientationBuilder(
      key: widget.key,
      builder: (context, orientation) {
        return LayoutBuilder(
          builder: (context, constraints) {
            return widget.builder(
              context,
              _getLayoutSize(
                orientation == Orientation.portrait
                    ? widget.portraitSizes
                    : widget.landscapeSizes,
                widget.useDeviceScreenSize
                    ? MediaQuery.of(context).size.width
                    : constraints.biggest.width,
              ),
              orientation,
            );
          },
        );
      },
    );
  }
}
